#!/usr/local/mascot/perl64/bin/perl

#############################################################################
# Copyright (C) 2021, Matrix Science Limited.                               #
#                                                                           #
# This script is free software. You can redistribute and/or                 #
# modify it under the terms of the GNU General Public License               #
# as published by the Free Software Foundation; either version 2            #
# of the License or, (at your option), any later version.                   #
#                                                                           #
# These modules are distributed in the hope that they will be useful,       #
# but WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the              #
# GNU General Public License for more details.                              #
#############################################################################

use strict;
use warnings;

# Default to Perl 5.8, which is the oldest version supported by Mascot
# 2.3 and 2.4.
use v5.008;

use File::Basename qw(fileparse);

if (not @ARGV) {
    print <<EOF;
Usage: divide_fasta_file.pl <FASTA file> [number of parts]

The script divides the FASTA file in the requested number of parts.
The parts are roughly even in size. If [number of parts] is not given,
default is 2.

Output files are written in the current directory and numbered after
the input file. For example, if input is "file.fasta", output files
are "file_1.fasta", "file_2.fasta", etc.

If you concatenate the output files, the result should be identical
to the input file.
EOF
    exit 0;
}

my $input_filepath = shift @ARGV;
my $number_of_parts = shift @ARGV;
$number_of_parts = 2 if not defined $number_of_parts;

if ($number_of_parts < 2 or $number_of_parts > 100) {
    die "Number of parts should be between 2 (inclusive) and 100 (inclusive)";
}

if (not -f $input_filepath) {
    die "'$input_filepath' does not exist or is not a file";
}

my $size_in_bytes = -s $input_filepath;

if (not defined $size_in_bytes or $size_in_bytes <= 0) {
    die "Cannot get size in bytes for '$input_filepath': $!";
}

my $output_filename_tmpl;
do {
    my ($filename, undef, $suffix) = fileparse($input_filepath, qr/\.[^.]*/);

    $output_filename_tmpl = $filename . '_%d' . $suffix;
};

my $chunk_size = $size_in_bytes / $number_of_parts;

# The output files won't be exactly $chunk_size, because we need to
# divide at a title line boundary. The excess should be no more than a
# few kilobytes (length of protein sequence).

open(my $fh_in, '<', $input_filepath) or die $!;

# Use binmode() to prevent line ending conversion.
binmode($fh_in);

my $n = 1;

my $output_filename = sprintf($output_filename_tmpl, $n);
print "Opening '$output_filename' for writing\n";

open(my $fh_out, '>', $output_filename) or die $!;

# Use binmode() to prevent line ending conversion.
binmode($fh_out);

my $bytes_written = 0;

while (<$fh_in>) {
    if (/^>/ and $bytes_written >= $chunk_size) {
        print "Finished writing $output_filename\n";
        close $fh_out or die $!;

        ++$n;

        $output_filename = sprintf($output_filename_tmpl, $n);
        print "Opening '$output_filename' for writing\n";

        open($fh_out, '>', $output_filename) or die $!;
        binmode($fh_out);

        $bytes_written = 0;
    }

    my $len = length($_);

    print $fh_out $_;
    
    $bytes_written += $len;
}

print "Finished writing $output_filename\n";
close $fh_out or die $!;
